CREATE TABLE device_type
(
    id_device_type serial NOT NULL,
    description character varying(100),
    PRIMARY KEY (id_device_type)
);

CREATE TABLE status
(
    id_status serial NOT NULL,
    description character varying(100),
    PRIMARY KEY (id_status)
);

CREATE TABLE device
(
    id_device serial NOT NULL,
    serial_number character varying(100),
    brand character varying(100),
    model character varying(100),
    id_device_type integer NOT NULL,
	status integer NOT NULL,
	last_inventory_check date NOT NULL DEFAULT now(),
    PRIMARY KEY (id_device),
    FOREIGN KEY (id_device_type)
        REFERENCES device_type (id_device_type) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
	FOREIGN KEY (status)
        REFERENCES status (id_status) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE supplier
(
    id_supplier serial NOT NULL,
    name character varying(100),
    address character varying(100),
    email character varying(100),
    telephone character varying(100),
    PRIMARY KEY (id_supplier)
);

CREATE TABLE investment_order
(
    id_investment_order serial NOT NULL,
    description character varying(100),
    date timestamp without time zone,
    PRIMARY KEY (id_investment_order)
);

CREATE TABLE orderx
(
    id_order serial NOT NULL,
    id_supplier integer,
    date_issued timestamp without time zone,
    id_investment_order integer,
    PRIMARY KEY (id_order),
    FOREIGN KEY (id_supplier)
        REFERENCES supplier (id_supplier) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_investment_order)
        REFERENCES investment_order (id_investment_order) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE order_line
(
    id_order integer NOT NULL,
    id_line integer NOT NULL,
	description character varying(100),
    price double precision,
	id_device integer,
	id_device_type integer NOT NULL,
    date_received timestamp without time zone,
    PRIMARY KEY (id_order, id_line),
    FOREIGN KEY (id_order)
        REFERENCES orderx (id_order) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_device)
        REFERENCES device (id_device) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
	FOREIGN KEY (id_device_type)
		REFERENCES public.device_type (id_device_type) MATCH SIMPLE
		ON UPDATE NO ACTION
		ON DELETE NO ACTION
		NOT VALID	
);

CREATE INDEX index_device
ON order_line(id_device);


CREATE TABLE financial_audit
(
    id_financial_audit serial NOT NULL,
    company character varying(100),
    total_checks integer NOT NULL DEFAULT 0,
    PRIMARY KEY (id_financial_audit)
);

CREATE TABLE inventory_check
(
    id_device integer NOT NULL,
    id_financial_audit integer NOT NULL,
    date timestamp without time zone NOT NULL,
    auditor character varying(100),
    comments character varying(300),
    PRIMARY KEY (id_Device, id_financial_audit, date),
    FOREIGN KEY (id_device)
        REFERENCES device (id_device) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_financial_audit)
        REFERENCES financial_audit (id_financial_audit) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE action_type
(
    id_action_type serial NOT NULL,
    description character varying(100),
    PRIMARY KEY (id_action_type)
);

CREATE TABLE department
(
    id_department serial NOT NULL,
    name character varying(100),
    head integer,
    users integer NOT NULL DEFAULT 0,
    last_year_incidents integer NOT NULL DEFAULT 0,
    inventory double precision NOT NULL DEFAULT 0,
	year_assigments integer NOT NULL DEFAULT 0;
    PRIMARY KEY (id_department)
);
	
CREATE TABLE usuari
(
    id_user serial NOT NULL,
    first_name character varying(100),
    last_name character varying(100),
    id_department integer NOT NULL,
    total_incidents integer NOT NULL DEFAULT 0,
    incidents_current_year integer NOT NULL DEFAULT 0,
    total_assigments integer NOT NULL DEFAULT 0,
    PRIMARY KEY (id_user),
    FOREIGN KEY (id_department)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

ALTER TABLE IF EXISTS department
    ADD FOREIGN KEY (head)
    REFERENCES usuari (id_user) MATCH SIMPLE
    ON UPDATE NO ACTION
    ON DELETE NO ACTION
    NOT VALID;

CREATE TABLE action
(
    id_device integer NOT NULL,
    id_user integer NOT NULL,
    date timestamp without time zone NOT NULL,
    id_action_type integer NOT NULL,
    responsable integer NOT NULL,
    PRIMARY KEY (id_device, id_user, date),
    FOREIGN KEY (id_device)
        REFERENCES device (id_device) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_user)
        REFERENCES usuari (id_user) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_action_type)
        REFERENCES action_type (id_action_type) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (responsable)
        REFERENCES usuari (id_user) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE incident
(
    id_incident serial NOT NULL,
    id_device integer NOT NULL,
    date timestamp without time zone NOT NULL,
	description char varying(100),
	technician integer NOT NULL,
    PRIMARY KEY (id_incident),
    FOREIGN KEY (id_device)
        REFERENCES device (id_device) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (technician)
        REFERENCES usuari (id_user) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);




CREATE TABLE process_audit
(
	id_process_audit serial NOT NULL,
    company character varying(100),
	total_checks integer NOT NULL DEFAULT 0,
	PRIMARY KEY (id_process_audit)
);


CREATE TABLE process_check
(
    id_process_audit integer NOT NULL,
    id_incident integer NOT NULL,
    date timestamp without time zone NOT NULL,
    auditor character varying(100),
    comments character varying(300),
    PRIMARY KEY (id_process_audit, id_incident, date),
    FOREIGN KEY (id_process_audit)
        REFERENCES process_audit (id_process_audit) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_incident)
        REFERENCES incident (id_incident) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE current_assigment
(
    id_device integer NOT NULL,
    id_user integer NOT NULL,
    PRIMARY KEY (id_device, id_user),
    FOREIGN KEY (id_device)
        REFERENCES device (id_device) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_user)
        REFERENCES usuari (id_user) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);
CREATE INDEX index_user
ON current_assigment(id_user);

CREATE TABLE budget
(
    id_department integer NOT NULL,
    year integer NOT NULL,
    aproved double precision,
	executed double precision,
    PRIMARY KEY (id_department, year),
    FOREIGN KEY (id_department)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE investment_order_year
(
    id_investment_order integer NOT NULL,
    year integer NOT NULL,
    ammount double precision,
    PRIMARY KEY (id_investment_order, year),
    FOREIGN KEY (id_investment_order)
        REFERENCES investment_order (id_investment_order) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);

CREATE TABLE devices_per_type
(
    id_department integer NOT NULL,
    id_device_type integer NOT NULL,
    units integer NOT NULL DEFAULT 0,
    PRIMARY KEY (id_department, id_device_type),
    FOREIGN KEY (id_department)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (id_device_type)
        REFERENCES device_type (id_device_type) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);
	
CREATE TABLE general_stats
(
    users integer NOT NULL DEFAULT 0,
    devices integer NOT NULL DEFAULT 0,
    laptops_no_trouble integer NOT NULL DEFAULT 0,
	laptops integer NOT NULL DEFAULT 0,
    old_inventory_devices integer NOT NULL DEFAULT 0,
    users_no_device integer NOT NULL DEFAULT 0,
    users_over5_devices integer NOT NULL DEFAULT 0,
    worst_ratio_department integer,
    most_device_types_dept integer,
	device_type_number integer NOT NULL DEFAULT 0,
    fastest_supplier integer,
	delivery_time interval NOT NULL DEFAULT '100 years'::interval,
    trouble_maker integer,
	laptop integer,
	it_department integer,
    FOREIGN KEY (worst_ratio_department)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (most_device_types_dept)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (fastest_supplier)
        REFERENCES supplier (id_supplier) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
    FOREIGN KEY (trouble_maker)
        REFERENCES usuari (id_user) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID,
	FOREIGN KEY (laptop)
    REFERENCES public.device_type (id_device_type) MATCH SIMPLE
    ON UPDATE NO ACTION
    ON DELETE NO ACTION
    NOT VALID,
	FOREIGN KEY (it_department)
    REFERENCES public.department (id_department) MATCH SIMPLE
    ON UPDATE NO ACTION
    ON DELETE NO ACTION
    NOT VALID
);

ALTER TABLE IF EXISTS general_stats
    ADD COLUMN id integer NOT NULL DEFAULT 1;
ALTER TABLE IF EXISTS general_stats
    ADD UNIQUE (id);
ALTER TABLE IF EXISTS general_stats
    ADD CHECK (id = 1)
    NOT VALID;
INSERT INTO general_stats (id) VALUES (1);

CREATE TABLE year_stats
(
    year integer NOT NULL,
    devices_bought integer NOT NULL DEFAULT 0,
    id_department integer,
    max_assigments integer NOT NULL DEFAULT 0,
    PRIMARY KEY (year),
    FOREIGN KEY (id_department)
        REFERENCES department (id_department) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
        NOT VALID
);
	
CREATE TABLE public.log
(
    id bigserial NOT NULL,
    taula character varying(50) NOT NULL,
    abm "char" NOT NULL,
    input character varying(400),
    rsp character varying(400) NOT NULL,
	date timestamp without time zone NOT NULL DEFAULT now(),
    CHECK (abm='a' or abm='b' or abm='m') NOT VALID
);

ALTER TABLE IF EXISTS public.log
    OWNER to postgres;