-- Procediments i Triggers ABM i Temporitzats

-- Alta a process_check

CREATE OR REPLACE FUNCTION new_process_check() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
    count integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN
    taula = 'process_check';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id_incident:',NEW.id_incident,' audit:',NEW.id_process_audit,' date:',NEW.date,' auditor:',NEW.auditor,' comments:',NEW.comments);

	SELECT total_checks into count FROM process_audit WHERE id_process_audit=NEW.id_process_audit;
	count = count + 1;
	UPDATE process_audit SET total_checks = count WHERE id_process_audit = NEW.id_process_audit;
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);		
			RETURN NULL;
END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_process_check
	AFTER INSERT
	ON process_check
	FOR EACH ROW
	EXECUTE FUNCTION new_process_check();




-- Alta a inventory_check

CREATE OR REPLACE FUNCTION new_inventory_check() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
    count integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN
    taula = 'inventory_check';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id_device:',NEW.id_device,' audit:',NEW.id_financial_audit,' date:',NEW.date,' auditor:',NEW.auditor,' comments:',NEW.comments);

	SELECT total_checks into count FROM financial_audit WHERE id_financial_audit=NEW.id_financial_audit;
	count = count + 1;
	UPDATE financial_audit SET total_checks = count WHERE id_financial_audit = NEW.id_financial_audit;
	
	UPDATE device SET last_inventory_check = NEW.date WHERE id_device = NEW.id_device; 
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);		
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;
	
END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_inventory_check
	AFTER INSERT
	ON inventory_check
	FOR EACH ROW
	EXECUTE FUNCTION new_inventory_check();



-- alta incident
CREATE OR REPLACE FUNCTION new_incident() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
    count1 integer;
	count2 integer;
	idx integer;
	idx2 integer;
	idx3 integer;
	date1 timestamp;
	date2 timestamp;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN

    taula = 'incident';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id_incident:',NEW.id_incident,' device:',NEW.id_device,' date:',NEW.date,' description:',NEW.description,' technician:',NEW.technician);

	-- verificar que responsable es tecnic
	SELECT id_department INTO idx FROM usuari WHERE id_user = NEW.technician;
	SELECT it_department INTO idx2 FROM general_stats WHERE id=1;
	IF idx != idx2 THEN
		RAISE EXCEPTION 'responsable no es tècnic';
	END IF;

	--- incrementar total incidents i incicents current year al usuari
    SELECT id_user INTO idx FROM current_assigment WHERE id_device=NEW.id_device;
    SELECT total_incidents,incidents_current_year into count1,count2 FROM usuari WHERE id_user = idx;
	count1 = count1 + 1;
	count2 = count2 + 1;
	UPDATE usuari SET total_incidents = count1, incidents_current_year = count2 WHERE id_user = idx;
	-- COALESCE ens torna el primer valor not null de la llista de parametres 
	SELECT incidents_current_year into count1 from usuari where id_user = COALESCE((SELECT trouble_maker from general_stats where id=1), idx);
	-- count2 pot ser = count 1 si el trouble maker era null o si, per casualitat tenen els mateixos incidents. per si de cas, el canvio.
	IF count2 >= count1 THEN
		UPDATE general_stats SET trouble_maker = idx WHERE id=1;
	END IF;
		

	-- disminueix laptops no trouble, si cal
	SELECT id_device_type INTO idx FROM device WHERE id_device = NEW.id_device;
	SELECT laptop, laptops_no_trouble INTO idx2, count1 FROM general_stats WHERE id = 1;
	-- device_type  = laptop
	IF idx = idx2 THEN
	    -- cal verificar si era o no un laptop _no_trouble
		SELECT id_user INTO idx3 FROM current_assigment WHERE id_device = NEW.id_device;
		SELECT date into date1 FROM action WHERE id_device = NEW.id_device and id_user = idx3 ORDER BY date DESC LIMIT 1;
		date2 = COALESCE ((SELECT date FROM incident WHERE id_device = NEW.id_device ORDER BY date DESC LIMIT 1),'2000-01-01 00:00:00');
		IF date2 < date1 THEN
			count1 = count1 -1;
			UPDATE general_stats SET laptops_no_trouble = count1 WHERE id=1;
		END IF;
	END IF;

	
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;
	
END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_incident
	BEFORE INSERT
	ON incident
	FOR EACH ROW
	EXECUTE FUNCTION new_incident();


-- modificacio order line
CREATE OR REPLACE FUNCTION update_order_line() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
    count1 integer;
	count2 integer;
	idx integer;
	temps1 interval;
    temps2 interval;
    dia date;
    mes_curt boolean;
	saldo double precision;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN

    taula = 'order_line';
    abm = 'm';
    rsp = 'OK';
    input = CONCAT('id_order:',NEW.id_order,' id_line:',NEW.id_line,' description:',NEW.description,' price:',NEW.price,' device_type:',NEW.id_device_type,' device:',NEW.id_device,' date:',NEW.date_received);

    -- alta de device i assigna id_device a order_line
	-- status 1 = pendent 
    INSERT INTO device(id_device_type,status,last_inventory_check) 
		VALUES	(OLD.id_device_type, 1, NEW.date_received) RETURNING id_device INTO NEW.id_device;

    -- incrementa devices a general_stats
	SELECT devices, laptops, laptop INTO count1, count2, idx FROM general_stats WHERE id=1;
	count1 = count1 +1;
	UPDATE general_stats SET devices = count1;


	
	-- incrementa devices_bought a year_stats
	SELECT date_part('year', NEW.date_received) :: integer INTO count1;
	SELECT devices_bought INTO count2 FROM year_stats WHERE year = count1;
	count2 = count2 + +1;
	UPDATE year_stats SET devices_bought = count2 WHERE year = count1;

	-- Actualitza fastest_supplier i delivery_time a general_stats si cal
	SELECT delivery_time INTO temps1 FROM general_stats WHERE id=1;
	SELECT date_issued INTO dia FROM orderx WHERE id_order = OLD.id_order;
	temps2 = NEW.date_received - dia;
    SELECT temps2 < temps1 INTO mes_curt;
	IF mes_curt THEN	
		SELECT id_supplier INTO idx FROM orderx WHERE id_order = OLD.id_order;
		UPDATE general_stats SET fastest_supplier=idx, delivery_time=temps2 WHERE id=1;
	END IF;		
	
	-- Incrementa ammount a investment_order_year si cal
	SELECT COALESCE((SELECT id_investment_order FROM orderx WHERE id_order= OLD.id_order),0) INTO idx;	
	IF idx>0 THEN
	    -- comprova si existeix el registre a investment_order_year, sino el crea
		SELECT COALESCE((SELECT ammount FROM investment_order_year 
		WHERE id_investment_order=idx and year=date_part('year', dia)::integer),0) INTO saldo;
	
		IF saldo > 0 THEN
			saldo = saldo + OLD.price;
			UPDATE investment_order_year SET ammount=saldo 
			WHERE id_investment_order=idx and year=date_part('year', dia)::integer;
		ELSE
			INSERT INTO investment_order_year(id_investment_order,year,ammount) 
			VALUES (idx,date_part('year', dia)::integer,OLD.price);
		END IF; 
	
	END IF;		

	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);		
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
            RETURN NULL;			
END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER update_order_line
	BEFORE UPDATE
	ON order_line
	FOR EACH ROW
	EXECUTE FUNCTION update_order_line();


-- alta order line
CREATE OR REPLACE FUNCTION new_order_line() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
	idx integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN
    taula = 'order_line';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id_order:',NEW.id_order,' id_line:',NEW.id_line,' description:',NEW.description,' price:',NEW.price,' device_type:',NEW.id_device_type);
	
	-- COALESCE ens torna el primer valor not null de la llista de parametres 
	SELECT COALESCE((SELECT id_line FROM order_line WHERE id_order=NEW.id_order ORDER BY id_line DESC LIMIT 1),0) 
	INTO idx;
	NEW.id_line = idx+1;	
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp); 	
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);	
			RETURN NULL;
	
END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_order_line
	BEFORE INSERT
	ON order_line
	FOR EACH ROW
	EXECUTE FUNCTION new_order_line();


-- alta action
CREATE OR REPLACE FUNCTION new_action() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
	idx integer;
	idx2 integer;
	idx3 integer;
	date1 timestamp;
	date2 timestamp;
	count1 integer;
	count2 integer;
	count3 integer;
	old_status integer;
	saldo double precision;
	preu double precision;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
BEGIN
    taula = 'action';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id_device:',NEW.id_device,' id_user:',NEW.id_user,' date:',NEW.date,' id_action_type:',NEW.id_action_type,' responsable:',NEW.responsable);
    -- verificar que responsable es tecnic
	SELECT id_department INTO idx FROM usuari WHERE id_user = NEW.responsable;
	SELECT it_department INTO idx2 FROM general_stats WHERE id=1;
	IF idx != idx2 THEN
		RAISE EXCEPTION 'responsable no es tècnic';
	END IF;
	
	-- llegir status actual del dispositiu
	SELECT status INTO old_status FROM device WHERE id_device = NEW.id_device;
	
	-- tipus 1 =  alta
	IF NEW.id_action_type = 1 THEN
		-- pasem status a 2, no assignat.
		UPDATE device SET status=2 WHERE id_device = NEW.id_device;
	END IF;
	
	-- tipus 2 =  assignació
	IF NEW.id_action_type = 2 THEN
		-- pasem status a 3, assignat.
		UPDATE device SET status=3 WHERE id_device = NEW.id_device;
		-- afegim registre a current_assigment
		INSERT INTO current_assigment(id_device,id_user) VALUES (NEW.id_device,NEW.id_user);
		-- incrementem tota_assigments al usuari 
		SELECT total_assigments INTO count1 FROM usuari WHERE id_user = NEW.id_user;
		count1 = count1 + 1;
		UPDATE usuari SET total_assigments = count1 WHERE id_user = NEW.id_user;
		-- augmentar usuaris amb mes de 5 o disminuir usuaris sense dispositiu si cal
		-- si son 6 en tenia 5 abans d'afegir aquest. si son més ja el teniem comptat
		IF count1 = 6 THEN
			SELECT users_over5_devices INTO count2 FROM general_stats WHERE id=1;
			count2 = count2 +1;
			UPDATE general_stats SET users_over5_devices = count2 WHERE id=1;
		END IF;
		-- si son 1 vol dir que abans no en tenia cap
		IF count1 = 1 THEN
			SELECT users_no_device INTO count2 FROM general_stats WHERE id=1;
			count2 = count2 - 1;
			UPDATE general_stats SET users_no_device = count2 WHERE id=1;			
		END IF;
		-- incrementar laptops i laptops_no_trouble si es un portatil
		SELECT id_device_type INTO idx FROM device WHERE id_device = NEW.id_device;
		SELECT laptop, laptops_no_trouble, laptops INTO idx2,count1,count2 FROM general_stats WHERE id=1;
		if idx = idx2 THEN
			count1 = count1 + 1;
			count2 = count2 + 1;
			UPDATE general_stats SET laptops_no_trouble = count1 , laptops = count2 WHERE id=1;	
		END IF;
		-- incrementar inventari departament (idx2) del usuari
		SELECT id_department INTO idx2 FROM usuari WHERE id_user = NEW.id_user;
		SELECT inventory INTO saldo FROM department WHERE id_department = idx2;
		SELECT price INTO preu FROM order_line WHERE id_device = NEW.id_device;
		saldo = saldo + preu;
		UPDATE department SET inventory = saldo WHERE id_department = idx2;
		-- incrementar units a devices_per_type  (tinc el device_type a idx)
		SELECT units INTO count1 FROM devices_per_type WHERE id_device_type=idx and id_department = idx2;
		count1 = count1 + 1;
		UPDATE devices_per_type SET units = count1 WHERE id_device_type=idx and id_department = idx2;
		-- Actualitzar most_device_types_dept i device_type_number a general_stats si cal
		SELECT device_type_number INTO count1 FROM general_stats WHERE id=1;
		SELECT COUNT(*) INTO count2 FROM devices_per_type WHERE id_department = idx2 AND units > 0;
		IF count2 > count1 THEN
			UPDATE general_stats SET device_type_number = count2 ,most_device_types_dept = idx2 WHERE id=1;
		END IF;	
		-- incrementar assignacions del departamenta a year_stats
		SELECT year_assigments INTO count1 FROM department WHERE id_department = idx2;
		count1 = count1+1;
		UPDATE department SET year_assigments = count1 where id_department = idx2;
		SELECT date_part('year', NEW.date) :: integer INTO count2;
		SELECT max_assigments INTO count3 FROM year_stats WHERE year =  count2;
		IF count1>count3 THEN
			UPDATE year_stats SET id_department=idx2, max_assigments=count1 WHERE year=count2;
		END IF;
		
	END IF;

	-- tipus 3 =  alliberament
	IF NEW.id_action_type = 3 THEN
		-- pasem status a 2, no-assignat.
		UPDATE device SET status=2 WHERE id_device = NEW.id_device;
	END IF;
		
	-- tipus 4 = baixa
	IF NEW.id_action_type = 4 THEN
		-- pasem status a 4, baixa.
		UPDATE device SET status=4 WHERE id_device = NEW.id_device;
		
 		-- decrementem devices a general_stats
		SELECT devices, laptops, laptop INTO count1,count2, idx2 FROM general_stats WHERE id=1;
		count1 = count1 - 1;
		UPDATE general_stats SET devices = count1 WHERE id=1;
		-- decrementem laptop si ho era 
		SELECT id_device_type INTO idx FROM device WHERE id_device = NEW.id_device;
		IF idx = idx2 THEN
			count2 = count2 - 1;
			UPDATE general_stats SET laptops = count2 WHERE id=1;	
		END IF;
	END IF;
	
	-- igual per alliberaments i per baixes (es per assignats)
	IF (NEW.id_action_type = 3 OR NEW.id_action_type = 4) AND old_status=3 THEN
		-- eliminem regisgre al current_assigment
		DELETE FROM current_assigment WHERE id_device = NEW.id_device;
		-- reduim total_assigments al usuari 
		SELECT total_assigments INTO count1 FROM usuari WHERE id_user = NEW.id_user;
		count1 = count1 - 1;
		UPDATE usuari SET total_assigments = count1 WHERE id_user = NEW.id_user;
	    -- reduir usuaris amb més de 5 o augmentar usuaris sense dispositiu si cal
		-- si son 5 en tenia 6 abans d'afegir aquest. si son menys ja el teniem comptat
		IF count1 = 5 THEN
			SELECT users_over5_devices INTO count2 FROM general_stats WHERE id=1;
			count2 = count2 - 1;
			UPDATE general_stats SET users_over5_devices = count2 WHERE id=1;
		END IF;
		-- si son 0 cal afegir
		IF count1 = 0 THEN
			SELECT users_no_device INTO count2 FROM general_stats WHERE id=1;
			count2 = count2 + 1;
			UPDATE general_stats SET users_no_device = count2 WHERE id=1;			
		END IF;
	    
		-- decrementar laptops_no_trouble si cal.  device_type = 1
		SELECT id_device_type INTO idx FROM device WHERE id_device = NEW.id_device;
		SELECT laptop, laptops_no_trouble INTO idx2, count1 FROM general_stats WHERE id= 1;
		IF idx = idx2 THEN
			-- cal verificar si era o no un laptop _no_trouble
			SELECT id_user INTO idx3 FROM current_assigment WHERE id_device = NEW.id_device;
			SELECT date into date1 FROM action WHERE id_device = NEW.id_device and id_user = idx3 ORDER BY date DESC LIMIT 1;
			date2 = COALESCE ((SELECT date FROM incident WHERE id_device = NEW.id_device ORDER BY date DESC LIMIT 1),'2000-01-01 00:00:00');
			IF date2 < date1 THEN
				count1 = count1 -1;
				UPDATE general_stats SET laptops_no_trouble = count1 WHERE id=1;
			END IF;
		END IF;
		
		
		-- decrementar inventari departament (idx2) del usuari
		SELECT id_department INTO idx2 FROM usuari WHERE id_user = NEW.id_user;
		SELECT inventory INTO saldo FROM department WHERE id_department = idx2;
		SELECT price INTO preu FROM order_line WHERE id_device = NEW.id_device;
		saldo = saldo - preu;
		UPDATE department SET inventory = saldo WHERE id_department = idx2;
		-- decrementar units a devices_per_type  (tinc el device_type a idx)
		SELECT units INTO count1 FROM devices_per_type WHERE id_device_type=idx and id_department = idx2;
		count1 = count1 - 1;
		UPDATE devices_per_type SET units = count1 WHERE id_device_type=idx and id_department = idx2;
		-- Actualitzar most_device_types_dept i device_type_number a general_stats si cal
		SELECT most_device_types_dept INTO idx3 FROM general_stats WHERE id=1;
		-- si no era el departament amb més tipus no caldra fer res.
		IF idx3 = idx2 THEN
		    SELECT id_department, COUNT(*) as c  INTO count1,count2 FROM devices_per_type WHERE units>0
			GROUP BY id_department
			ORDER BY c DESC
			LIMIT 1;
			UPDATE general_stats SET device_type_number = count2 WHERE id=1;
			UPDATE general_stats SET most_device_types_dept = count1 WHERE id=1;
		END IF;
	END IF;

    INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp); 	
	RETURN NEW;
	
	
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;

END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_action
	BEFORE INSERT
	ON action
	FOR EACH ROW
	EXECUTE FUNCTION new_action();



-- Alta a departament

CREATE OR REPLACE FUNCTION new_department() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;	
    idx integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;

BEGIN
    taula = 'department';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('name:',NEW.name,' head:',NEW.head,' users:',NEW.users,' lyi:',NEW.last_year_incidents,' inventory:',NEW.inventory);
    FOR idx IN (SELECT id_device_type FROM device_type)
    LOOP
	    INSERT INTO devices_per_type(id_department,id_device_type) VALUES (NEW.id_department,idx)
	    ON CONFLICT DO NOTHING;
	END LOOP;
    
    INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
            INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;
END;

$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_department
	AFTER INSERT
	ON department
	FOR EACH ROW
	EXECUTE FUNCTION new_department();



-- Alta a device_type

CREATE OR REPLACE FUNCTION new_device_type() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
	idx integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
	
BEGIN
    taula = 'device_type';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id:',NEW.id_device_type,' description:',NEW.description);
    FOR idx IN (SELECT id_department FROM department)
    LOOP
	    INSERT INTO devices_per_type(id_department,id_device_type) VALUES (idx,NEW.id_device_type)
	    ON CONFLICT DO NOTHING;
	END LOOP;
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
			INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;
            


END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_device_type
	AFTER INSERT
	ON device_type
	FOR EACH ROW
	EXECUTE FUNCTION new_device_type();



-- Alta a usuari

CREATE OR REPLACE FUNCTION new_user() 
  RETURNS TRIGGER AS
$$
DECLARE
	text1 varchar;
	text2 varchar;
	text3 varchar;
	count1 integer;
	count2 integer;
	input varchar;
    taula varchar;
    abm varchar;
	rsp varchar;
	
BEGIN
    taula = 'usuari';
    abm = 'a';
    rsp = 'OK';
    input = CONCAT('id:',NEW.id_user,' name:',NEW.first_name,' last_name:',NEW.last_name,' department:',NEW.id_department);
	-- afegeix usuari i usuari sense dispositiu
	SELECT users,users_no_device INTO count1,count2 FROM general_stats WHERE id=1;
	count1 = count1 + 1;
	count2 = count2 + 1;
	UPDATE general_stats SET users = count1, users_no_device = count2 WHERE id=1;	
    -- incrementa users a department
 	SELECT users INTO count1 FROM department WHERE id_department=NEW.id_department;
	count1 = count1 + 1;
	UPDATE department SET users = count1 WHERE id_department = NEW.id_department;

 
 
 
	INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
	RETURN NEW;
	
	EXCEPTION
		WHEN others THEN
			GET STACKED DIAGNOSTICS
              	text1 = RETURNED_SQLSTATE,
                text2 = PG_EXCEPTION_DETAIL;
            rsp = CONCAT('ERROR:',text1,' ',text2);
			INSERT INTO log(taula,abm,input,rsp) VALUES (taula,abm,input,rsp);
			RETURN NULL;
            


END;
$$ LANGUAGE PLPGSQL;

CREATE OR REPLACE TRIGGER new_user
	AFTER INSERT
	ON usuari
	FOR EACH ROW
	EXECUTE FUNCTION new_user();

	
-- Procediment anual
CREATE OR REPLACE PROCEDURE cada_any()
LANGUAGE plpgsql
AS 
$$
BEGIN
-- crear un nou registre a year_stats
INSERT INTO year_stats(year) SELECT date_part('year', CURRENT_DATE) :: integer;
UPDATE usuari SET incidents_current_year=0;
UPDATE department SET year_assigments=0;
END;
$$;

-- Scheduled job del primer dia de l'any	
DO $$
DECLARE
    jid integer;
    scid integer;
BEGIN
-- Creating a new job
INSERT INTO pgagent.pga_job(
    jobjclid, jobname, jobdesc, jobhostagent, jobenabled
) VALUES (
    1::integer, 'cada_any'::text, 'el primer dia de cada any, crea un registre nou a year_stats i posa a zero el incidents_current_year de tots els usuaris'::text, ''::text, true
) RETURNING jobid INTO jid;

-- Steps
-- Inserting a step (jobid: NULL)
INSERT INTO pgagent.pga_jobstep (
    jstjobid, jstname, jstenabled, jstkind,
    jstconnstr, jstdbname, jstonerror,
    jstcode, jstdesc
) VALUES (
    jid, 'pas1'::text, true, 's'::character(1),
    ''::text, 'postgres'::name, 'f'::character(1),
    'CALL cada_any();'::text, ''::text
) ;

-- Schedules
-- Inserting a schedule
INSERT INTO pgagent.pga_schedule(
    jscjobid, jscname, jscdesc, jscenabled,
    jscstart,     jscminutes, jschours, jscweekdays, jscmonthdays, jscmonths
) VALUES (
    jid, 'capdany'::text, ''::text, true,
    '2021-12-06 12:38:03 +01:00'::timestamp with time zone, 
    -- Minutes
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Hours
    ARRAY[false,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Week days
    ARRAY[false,false,false,false,false,false,false]::boolean[],
    -- Month days
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Months
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false]::boolean[]
) RETURNING jscid INTO scid;
END
$$;


-- Procediment diari	
CREATE OR REPLACE PROCEDURE cada_dia()
LANGUAGE plpgsql
AS 
$$
BEGIN
UPDATE general_stats SET old_inventory_devices = (SELECT COUNT(*) from device where now()>last_inventory_check + interval '1 year');
END;
$$;

-- Scheduled job diari	
DO $$
DECLARE
    jid integer;
    scid integer;
BEGIN
-- Creating a new job
INSERT INTO pgagent.pga_job(
    jobjclid, jobname, jobdesc, jobhostagent, jobenabled
) VALUES (
    1::integer, 'cada_dia'::text, 'actualitza el registre old_inventory_devices amb els dispositius que fa mes d''un any que no s''han auditat'::text, ''::text, true
) RETURNING jobid INTO jid;

-- Steps
-- Inserting a step (jobid: NULL)
INSERT INTO pgagent.pga_jobstep (
    jstjobid, jstname, jstenabled, jstkind,
    jstconnstr, jstdbname, jstonerror,
    jstcode, jstdesc
) VALUES (
    jid, 'pas1'::text, true, 's'::character(1),
    ''::text, 'postgres'::name, 'f'::character(1),
    'CALL cada_dia();'::text, ''::text
) ;

-- Schedules
-- Inserting a schedule
INSERT INTO pgagent.pga_schedule(
    jscjobid, jscname, jscdesc, jscenabled,
    jscstart,     jscminutes, jschours, jscweekdays, jscmonthdays, jscmonths
) VALUES (
    jid, 'diari'::text, ''::text, true,
    '2021-12-06 13:05:13 +01:00'::timestamp with time zone, 
    -- Minutes
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Hours
    ARRAY[false,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Week days
    ARRAY[true,true,true,true,true,true,true]::boolean[],
    -- Month days
    ARRAY[false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Months
    ARRAY[false,false,false,false,false,false,false,false,false,false,false,false]::boolean[]
) RETURNING jscid INTO scid;
END
$$;


-- Procediment mensual	
CREATE OR REPLACE PROCEDURE cada_mes()
LANGUAGE plpgsql
AS 
$$
BEGIN
UPDATE department AS dp SET last_year_incidents =(SELECT COUNT(*) FROM incident 
INNER JOIN current_assigment
ON incident.id_device = current_assigment.id_device
INNER JOIN usuari
ON current_assigment.id_user = usuari.id_user
INNER JOIN department
ON usuari.id_department = department.id_department
WHERE department.id_department = dp.id_department AND now()<incident.date + interval '1 year'	);


UPDATE general_stats SET worst_ratio_department = (SELECT id_department from department 
ORDER BY (last_year_incidents) :: double precision / (users) :: double precision DESC
LIMIT 1);

END;
$$;

-- Scheduled job mensual
DO $$
DECLARE
    jid integer;
    scid integer;
BEGIN
-- Creating a new job
INSERT INTO pgagent.pga_job(
    jobjclid, jobname, jobdesc, jobhostagent, jobenabled
) VALUES (
    1::integer, 'cada_mes'::text, ''::text, ''::text, true
) RETURNING jobid INTO jid;

-- Steps
-- Inserting a step (jobid: NULL)
INSERT INTO pgagent.pga_jobstep (
    jstjobid, jstname, jstenabled, jstkind,
    jstconnstr, jstdbname, jstonerror,
    jstcode, jstdesc
) VALUES (
    jid, 'pas1'::text, true, 's'::character(1),
    ''::text, 'postgres'::name, 'f'::character(1),
    'CALL cada_mes();'::text, ''::text
) ;

-- Schedules
-- Inserting a schedule
INSERT INTO pgagent.pga_schedule(
    jscjobid, jscname, jscdesc, jscenabled,
    jscstart,     jscminutes, jschours, jscweekdays, jscmonthdays, jscmonths
) VALUES (
    jid, 'mensual'::text, ''::text, true,
    '2021-12-06 13:22:36 +01:00'::timestamp with time zone, 
    -- Minutes
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Hours
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Week days
    ARRAY[false,false,false,false,false,false,false]::boolean[],
    -- Month days
    ARRAY[true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false]::boolean[],
    -- Months
    ARRAY[true,true,true,true,true,true,true,true,true,true,true,true]::boolean[]
) RETURNING jscid INTO scid;
END
$$;